﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum Direction
{
    BottomLeft, Bottom, BottomRight, Left, Right, TopLeft, Top, TopRight
}

public class Neuron : MonoBehaviour
{
    public enum NeuronType
    {
        Normal, Start, End
    }

    public ColorSetter neuronColor;
    public ColorSetter neuronGlowColor;

    [HideInInspector]
    public NeuronType neuronType;

    [HideInInspector]
    public Neuron[] neighbours = new Neuron[8];

    Synapse[] synapses = new Synapse[8];

    public Neuron GetNeighbour(Direction dir)
    {
        if (neighbours == null || neighbours.Length < 8)
            neighbours = new Neuron[8];
        return neighbours[(int)dir];
    }

    public void SetNeighbour(Direction dir, Neuron n)
    {
        if (neighbours == null || neighbours.Length < 8)
            neighbours = new Neuron[8];

        if (neighbours[(int)dir] != null)
        {
            if (synapses[(int)dir] != null)
                Destroy(synapses[(int)dir].gameObject);
            neighbours[(int)dir] = null;
        }

        if (n == null)
            return;

        if (n.neighbours == null || n.neighbours.Length < 8)
            n.neighbours = new Neuron[8];

        neighbours[(int)dir] = n;

        var op = getOpposite(dir);
        n.neighbours[(int)op] = this;

        //Create synapse
        var go = Instantiate(ReferenceManager.instance.synapsePrefab);
        var syn = go.GetComponent<Synapse>();
        syn.Init(n, this);
        synapses[(int)dir] = syn;
    }

    public void Destroy()
    {
        foreach (var item in synapses)
        {
            if (item != null)
                Destroy(item.gameObject);
        }

        for (int i = 0; i < 8; i++)
        {
            var dir = (Direction)i;
            var op = getOpposite(dir);

            var n = GetNeighbour(dir);
            if (n == null)
                continue;
            var opn = GetNeighbour(op);
            n.SetNeighbour(op, opn);
        }

        Destroy(gameObject);
    }

    Direction getOpposite(Direction dir)
    {
        return (Direction)(7 - (int)dir);
    }

    public void SetNeuronType(NeuronType nt)
    {
        this.neuronType = nt;
        if (neuronType == NeuronType.Normal)
            neuronColor.color = ColorPallete.NormalNeuron;
        else if (neuronType == NeuronType.Start){
            neuronColor.color = ColorPallete.StartNeuron;
            neuronGlowColor.color = ColorPallete.StartNeuron;
        }
        else if (neuronType == NeuronType.End){
            neuronColor.color = ColorPallete.EndNeuron;
            neuronGlowColor.color = ColorPallete.EndNeuron;
        }
        neuronColor.Refresh();
        neuronGlowColor.Refresh();
    }

    public List<Vector2> GetPath(Neuron neighbour)
    {                   
        var lst = new List<Vector2>();
        if (neighbour == null)
            return lst;

        //Find direction
        var ns = new List<Neuron>(neighbours);
        int index = ns.IndexOf(neighbour);
        if (index >= 0)
        {
            var dir = (Direction)index;
            //Get related synapse
            var syn = synapses[index] ?? neighbour.synapses[(int)getOpposite(dir)];
            if (syn != null)
            {
                Vector3[] ps = new Vector3[syn.lineRend.positionCount];
                syn.lineRend.GetPositions(ps);
                bool reverse = false;
                if (syn.startNeuron == neighbour)
                    reverse = true;

                var tmp = new LinkedList<Vector2>();
                foreach (var p in ps)
                {
                    if (reverse)
                        tmp.AddFirst(p + syn.transform.position);
                    else
                        tmp.AddLast(p + syn.transform.position);
                }
                lst.AddRange(tmp);
            }
            else
            {
                lst.Add(neighbour.transform.position);
            }
        }
        else
        {
            lst.Add(neighbour.transform.position);
        }

        return lst;
    }
}
