﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Pulse : MonoBehaviour {
    public float dischargeRate = 0.1f;
    public float chargeAmount = 0.5f;
    public float minEnergyThreshold = 0.1f;
    public float energy;
    public SpriteRenderer energySpren;
    public SpriteRenderer glowSpren;
    public float speed = 3;

    public void setEnergy(float val){
        Color c = glowSpren.color;
        c.a = val / 2f;
        glowSpren.color = c;

        c = energySpren.color;
        c.a = val + 0.2f;
        energySpren.color = c;

        energy = val;
        transform.localScale = Vector3.one * val;
        if(energy > 1)
        {
            //Overcharged
            GameObject tmp = Instantiate(ReferenceManager.instance.pulseDestory,transform.position,transform.rotation);
            SpriteRenderer sr = tmp.GetComponent<SpriteRenderer>();
            Color src = sr.color;
            var minScale = Vector3.one * 0.1f;
            var maxScale = Vector3.one * 1.2f;
            ReferenceManager.instance.StartCoroutine(Utils.DoOverTime(1f, (t) =>
            {
                tmp.transform.localScale = Vector3.Lerp(minScale, maxScale, t);
                src.a = Mathf.Lerp(1, 0, t);
                sr.color = src;
            }, () =>
            {
                Destroy(tmp);
            }));
            Destroy(gameObject);
        }

    }

    public void Charge()
    {
        setEnergy(energy+chargeAmount);
    }

    private void FixedUpdate()
    {
        if (energy >= minEnergyThreshold)
        {
            setEnergy(energy - dischargeRate);
        }
        else
        {
            //Min energy, destroy and inform maybe
            Vector3 sc = transform.localScale;
            speed = 0.0001f;
            StartCoroutine(Utils.DoOverTime(0.5f, (t) =>
            {
                transform.localScale = Vector3.Lerp(sc, Vector3.zero, t);
            }, () =>
            {
                Destroy(gameObject);
            }));
        }
    }

    public void FollowPath(List<Neuron> path)
    {
        StartCoroutine(follow(path));
    }

    IEnumerator follow(List<Neuron> npath)
    {
        if (npath.Count > 0)
        {
            Neuron n = null;

            foreach (var np in npath)
            {
                if (n == null)
                {
                    //First neuron, we are there already
                    n = np;
                    continue;
                }
                var path = n.GetPath(np);

                foreach (var p in path)
                {
                    yield return StartCoroutine(Utils.MoveItem(transform, p, (p - (Vector2)transform.position).sqrMagnitude / speed));
                }

                n = np;
            }
        }

        ReferenceManager.instance.gameController.HandleWin();
    }


}
