﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Synapse : MonoBehaviour
{    
    public LineRenderer lineRend;
    public float segmentSize = 0.1f;
    public float offsetSize = 1;

    [HideInInspector]
    public Neuron startNeuron;
    [HideInInspector]
    public Neuron endNeuron;

    private void Awake()
    {
        lineRend.sortingLayerName = "Synapse";
    }

    public void Init(Neuron start, Neuron end)
    {
        this.startNeuron = start;
        this.endNeuron = end;
        List<Vector3> lst = new List<Vector3>();
        transform.position = start.transform.position;
        Vector3 endPoint = end.transform.position - transform.position;
        lst.Add(Vector3.zero);

        if (segmentSize > 0)
        {
            int count = Mathf.CeilToInt(endPoint.magnitude / segmentSize);
            var normal = new Vector2(endPoint.y, -endPoint.x).normalized;
            float currentOffset = 0;
            bool sin = Random.Range(0, 1f) < 0.5f;
            for (int i = 0; i < count; i++)
            {
                float o = (sin ? Mathf.Sin(i / 3f) : Mathf.Cos(i / 3f)) * offsetSize;
                var pos = Vector2.Lerp(Vector2.zero, endPoint, (float)i / count);
                //Make offset perpendicular to the pos line
                currentOffset = currentOffset * 0.95f + o * 0.05f;
                pos += normal * currentOffset;
                lst.Add(pos);
            }
        }

        lst.Add(endPoint);


        lineRend.positionCount = lst.Count;
        lineRend.SetPositions(lst.ToArray());
    }
}
