﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Audio;

public class AudioManager : MonoBehaviour
{
    private static AudioManager instance;
    public static AudioManager Instance { get { if (instance == null) { instance = FindObjectOfType<AudioManager>(); } return instance; } }

    [Header("SFX")]
    public AudioSource click;
    public AudioSource winGame, loseGame;

    [Header("Musics")]
	public AudioSource music;

    List<AudioSource> fadingSources = new List<AudioSource>();

    const float fadeTime = 1f;

    List<string> playlists = new List<string>();
    List<string> playlistsPlaying = new List<string>();

    void Awake()
    {
        instance = this;
    }

    void playAudio(AudioSource source, bool fade = false)
    {
        if (source != null && source.clip != null && !source.isPlaying)
        {
            if (fade)
            {
                StartCoroutine(Utils.DoAfterCondition(() =>
                {
                    fadingSources.Add(source);
                    float vol = source.volume;
                    source.volume = 0;
                    source.Play();
                    StartCoroutine(Utils.DoOverTime(fadeTime, (dt) =>
                    {
                        source.volume = Mathf.Lerp(0, vol, dt);
                    }, () =>
                    {
                        //onEnd
                        source.volume = vol;
                        fadingSources.Remove(source);
                    }));
                }, () => { return !fadingSources.Contains(source); }));
            }
            else
            {
                source.Play();
            }
        }
    }    

    void stopAudio(AudioSource source, bool fade = true)
    {
        if (source != null && source.clip != null && source.isPlaying)
        {
            if (fade)
            {
                StartCoroutine(Utils.DoAfterCondition(() =>
                {
                    fadingSources.Add(source);
                    float vol = source.volume;
                    StartCoroutine(Utils.DoOverTime(fadeTime, (dt) =>
                    {
                        source.volume = Mathf.Lerp(vol, 0, dt);
                    }, () =>
                    {
                        //onEnd
                        source.Stop();
                        source.volume = vol;
                        fadingSources.Remove(source);
                    }));
                }, () => { return !fadingSources.Contains(source); }));
            }
            else
            {
                source.Stop();
            }
        }
    }


    IEnumerator doCrossFade(AudioSource sourceOut, AudioSource sourceIn)
    {
        float volIn = sourceIn != null ? sourceIn.volume : 0;
        float volOut = sourceOut != null ? sourceOut.volume : 0;

        if (sourceIn != null)
        {
            sourceIn.volume = 0;
            sourceIn.Play();
        }
        yield return StartCoroutine(Utils.DoOverTime(fadeTime, (dt) =>
        {
            if (sourceIn != null)
                sourceIn.volume = Mathf.Lerp(0, volIn, dt);
            if (sourceOut != null)
                sourceOut.volume = Mathf.Lerp(volOut, 0, dt);
        }));
        if (sourceOut != null)
        {
            sourceOut.Stop();
            sourceOut.volume = volOut;
        }
        if (sourceIn != null)
            sourceIn.volume = volIn;
    }

    void playListLooped(string name, List<AudioSource> list, bool crossFade = true)
    {        
        StartCoroutine(playList(name, list, crossFade));
    }

    void stopListLooped(string name)
    {
        if (playlists.Contains(name))
            playlists.Remove(name);
    }

    IEnumerator playList(string name, List<AudioSource> list, bool crossFade = true)
    {
        while (playlistsPlaying.Contains(name))
            yield return null;
        playlists.Add(name);
        playlistsPlaying.Add(name);
        AudioSource current = null;
        while (playlists.Contains(name))
        {
            for (int i = 0; i < list.Count; i++)
            {
                if (crossFade)
                    yield return StartCoroutine(doCrossFade(current, list[i]));
                else
                {
                    if (current != null)
                        current.Stop();
                    list[i].Play();
                }
                current = list[i];

                if (crossFade)
                {
                    while ((current.clip.length - current.time) > fadeTime)
                    {
                        if (!playlists.Contains(name))
                            break;
                        yield return null;
                    }
                }
                else
                {
                    while (current.isPlaying)
                    {
                        if (!playlists.Contains(name))
                            break;
                        yield return null;
                    }
                }
            }
            yield return null;
        }
        if (current != null)
        {
            yield return StartCoroutine(doCrossFade(current, null));
        }
        if (playlists.Contains(name))
            playlists.Remove(name);
        playlistsPlaying.Remove(name);
    }

    public static void PlayClick() { if (Instance != null) Instance.playAudio(Instance.click); }
    public static void PlayWinGame() { if (Instance != null) Instance.playAudio(Instance.winGame); }

    public static void PlayMenuMusic() { if (Instance != null) Instance.playListLooped("menuMusic", new List<AudioSource>() { instance.music }, false); }
    public static void StopMenuMusic() { if (Instance != null) Instance.stopListLooped("menuMusic"); }
}
