﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine.UI;
using UnityEngine;

public class ColorSetter : MonoBehaviour
{
    public static Color GetColor(ColorPallete col, ColorPallete defaultColor = ColorPallete.BGColor)
    {
        if (Pallete.ContainsKey(col))
            return Pallete[col];
        else if (Pallete.ContainsKey(defaultColor))
            return Pallete[defaultColor];
        return Color.white;
    }

    public static ColorPallete GetColorSafe(int colorId, ColorPallete defaultColor = ColorPallete.BGColor)
    {
        ColorPallete col;
        try { col = (ColorPallete)colorId; }
        catch { col = defaultColor; }
        return col;
    }

    public static Color CodeToColor(string rgbCode)
    {
        Color c = new Color();
        c.r = ((float)int.Parse(rgbCode.Substring(0, 2), System.Globalization.NumberStyles.HexNumber)) / 255f;
        c.g = ((float)int.Parse(rgbCode.Substring(2, 2), System.Globalization.NumberStyles.HexNumber)) / 255f;
        c.b = ((float)int.Parse(rgbCode.Substring(4, 2), System.Globalization.NumberStyles.HexNumber)) / 255f;
        c.a = 1;
        return c;
    }

    public static string ColorToCode(Color col, bool hasAlpha = true)
    {
        var s = "";
        s += Mathf.Clamp(Mathf.RoundToInt(col.r * 255), 0, 255).ToString("X2");
        s += Mathf.Clamp(Mathf.RoundToInt(col.g * 255), 0, 255).ToString("X2");
        s += Mathf.Clamp(Mathf.RoundToInt(col.b * 255), 0, 255).ToString("X2");
        if (hasAlpha)
            s += Mathf.Clamp(Mathf.RoundToInt(col.a * 255), 0, 255).ToString("X2");
        return s;
    }

    public static Dictionary<ColorPallete, Color> Pallete = new Dictionary<ColorPallete, Color>()
    {
        {ColorPallete.BGColor, CodeToColor("20013dFF")},
        {ColorPallete.UIText, CodeToColor("F6D15EFF")},
        {ColorPallete.UITextDark, CodeToColor("54311B")},
        {ColorPallete.UITextDisable, CodeToColor("A37D06")},
        {ColorPallete.StartNeuron, CodeToColor("90fcf9FF")},
        {ColorPallete.EndNeuron, CodeToColor("90fcf9FF")},
        {ColorPallete.NormalNeuron, CodeToColor("cc8ac4FF")},
        {ColorPallete.NeuronGlow, CodeToColor("cc8ac4FF")},
        {ColorPallete.Synapse, CodeToColor("AAAAAA")},
        {ColorPallete.Pulse, CodeToColor("90fcf9FF")},
        {ColorPallete.PulseGlow, CodeToColor("51e5ffFF")},
    };

    public ColorPallete color;
    public Component target;

    public void Refresh()
    {
        if (target == null)
        {
            target = GetComponent<Graphic>();
            if (target == null)
                return;
        }

#if UNITY_EDITOR
        UnityEditor.Undo.RecordObject(this, "Set Color");
#endif

        if (target is SpriteRenderer)
            ((SpriteRenderer)target).color = Pallete[color];
        if (target is Graphic)
            ((Graphic)target).color = Pallete[color];
        else if (target is Shadow)
            ((Shadow)target).effectColor = Pallete[color];
        else if (target is TrailRenderer)
        {
            Color c = Pallete[color];
            c.a = 0.5f;
            ((TrailRenderer)target).startColor = c;
            c.a = 0;
            ((TrailRenderer)target).endColor = c;
        }
        else if (target is LineRenderer)
        {
            Color c = Pallete[color];
            var lr = target as LineRenderer;
            lr.startColor = c;
            lr.endColor = c;
        }
    }

#if UNITY_EDITOR
    public static List<T> FindAssetsByType<T>() where T : UnityEngine.Object
    {
        List<T> assets = new List<T>();
        string[] guids = UnityEditor.AssetDatabase.FindAssets(string.Format("t:{0}", typeof(T)).Replace("UnityEngine.", ""));
        for (int i = 0; i < guids.Length; i++)
        {
            string assetPath = UnityEditor.AssetDatabase.GUIDToAssetPath(guids[i]);
            T asset = UnityEditor.AssetDatabase.LoadAssetAtPath<T>(assetPath);
            if (asset != null)
            {
                assets.Add(asset);
            }
        }
        return assets;
    }

    [UnityEditor.MenuItem("Tools/Update Color Pallete")]
    public static void ChangePallete()
    {
        int count = 0;
        var items = FindObjectsOfType<ColorSetter>();
        count += items.Length;
        foreach (var item in items)
        {
            item.Refresh();
        }

        items = FindAssetsByType<ColorSetter>().ToArray();
        count += items.Length;
        foreach (var item in items)
        {
            item.Refresh();
        }
        UnityEditor.AssetDatabase.SaveAssets();
        print(count + " items updated.");
    }
#endif

}

public enum ColorPallete
{
    BGColor = 0, UIText = 1, UITextDark = 2,
    UITextDisable = 5,

    StartNeuron = 20, EndNeuron = 21, NormalNeuron = 22, Synapse = 23, Pulse = 24, PulseGlow = 25, NeuronGlow=26,
}