﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Dialog : MonoBehaviour
{
    public Text dialogText;
    public GameObject buttonPrefab;

    public class ButtonElement
    {
        public string text;
        public Action action;

        public ButtonElement(string buttonText, Action action = null)
        {
            this.text = buttonText ?? "";
            this.action = action;
        }
    }

    public void Init(string txt, params ButtonElement[] buttons)
    {
        dialogText.text = txt;
        foreach (var button in buttons)
        {
            var go = Instantiate(buttonPrefab, buttonPrefab.transform.parent, false);
            go.SetActive(true);
            var b = go.GetComponent<Button>();
            b.onClick.AddListener(() =>
            {
                if (button.action != null)
                    button.action();
                Destroy(this.gameObject);
            });
            b.GetComponentInChildren<Text>().text = button.text;
        }
    }

    public void Destroy()
    {
        Destroy(gameObject);
    }

    static DialogContainer dialogContainer;

    public static Dialog Open(string txt, params ButtonElement[] buttons)
    {
        if (dialogContainer == null)
        {
            dialogContainer = FindObjectOfType<DialogContainer>();
        }
        if (dialogContainer != null)
        {
            var dialog = Instantiate(dialogContainer.dialogPrefab, dialogContainer.transform, false).GetComponent<Dialog>();
            dialog.gameObject.SetActive(true);
            dialog.Init(txt, buttons);
            dialog.transform.SetAsLastSibling();
            return dialog;
        }
        return null;
    }

    public static void OpenTimed(string txt, float time)
    {
        if (dialogContainer == null)
        {
            dialogContainer = FindObjectOfType<DialogContainer>();
        }
        if (dialogContainer != null)
        {
            var dialog = Instantiate(dialogContainer.dialogPrefab, dialogContainer.transform, false).GetComponent<Dialog>();
            dialog.gameObject.SetActive(true);
            dialog.Init(txt, new ButtonElement[0]);
            dialog.transform.SetAsLastSibling();

            dialogContainer.StartCoroutine(Utils.DoDelayed(time, () => Destroy(dialog.gameObject)));
        }
    }
}
